#!/bin/sh
# wan_snmp_install.sh	WANPIPE SNMP Installation Script.
#
# Copyright	(c) 2004, Sangoma Technologies Inc.
#
#		This program is free software; you can redistribute it and/or
#		modify it under the terms of the GNU General Public License
#		as published by the Free Software Foundation; either version
#		2 of the License, or (at your option) any later version.
# ----------------------------------------------------------------------------
# Jan 20, 2004	Alex Feldman	Initial version.
# ============================================================================

####### FUNCTION DEFINITIONS #################################################

# ----------------------------------------------------------------------------
# Clear the screen if it is supported.
# ----------------------------------------------------------------------------
clearscr()
{
	if test $NONINTERACTIVE; then
		return
	fi

	# check if the terminal environment is set up
	[ "$TERM" ] && clear 2> /dev/null
}

check_bash ()
{
	BASH_SUPPORT=`echo $BASH_VERSION | cut -d'.' -f1 2> /dev/null`
}


# ----------------------------------------------------------------------------
# Display error message.
# ----------------------------------------------------------------------------
error()
{
	echo -e "Error: $*!" >&2
	return 0
}

# ----------------------------------------------------------------------------
# Pause.
# ----------------------------------------------------------------------------
pause()
{

	if test $NONINTERACTIVE; then
		return 0
	fi

	[ $# -ne 0 ] && echo -e $* >&2
	echo -e "Press [Enter] to continue...\c" >&2
	read tmp
	return 0
}

# ----------------------------------------------------------------------------
# Prompt user for input.
# ----------------------------------------------------------------------------
prompt()
{
	if test $NONINTERACTIVE; then
		return 0
	fi

	echo -ne "$*" >&2
	read CMD rest
	return 0
}

# ----------------------------------------------------------------------------
# Get Yes/No
# ----------------------------------------------------------------------------
getyn()
{
	if test $NONINTERACTIVE; then
		return 0
	fi

	while prompt "$* (y/n) "
	do	case $CMD in
			[yY])	return 0
				;;
			[nN])	return 1
				;;
			*)	echo -e "\nPlease answer y or n" >&2
				;;
		esac
	done

}

# ----------------------------------------------------------------------------
# Display banner.
# ----------------------------------------------------------------------------
banner()
{
	clearscr
	echo -e "\t----------------------------------------------------------"
	echo -e "\t          WANPIPe SNMP Installation Script"
	echo -e "\t    Copyright (c) 2004, Sangoma Technologies Inc."
	echo -e "\t----------------------------------------------------------"
	echo ""
	return 0
}


function init_all_mibs ()
{
	FR_MIB=NO
	PPP_MIB=NO
	DS1_MIB=NO
	X25_MIB=NO

	WANPIPE_SNMP_MIBS=""
}

function enable_snmp_mibs ()
{
	local MIB_MATCH=NO
	local MIB=$1
	local ARG

	ARG=${2:-NO}
	
	echo "$MIB" | grep "FR" > /dev/null	
	if [ $? -eq 0 ]; then
		if [ $FR_MIB != YES ]; then
			echo "Enabling the FR SNMP mib"
			WANPIPE_SNMP_MIBS="$WANPIPE_SNMP_MIBS frameRelayDTE"
		fi
		MIB_MATCH=YES	
		FR_MIB=YES
	fi
	
	echo "$MIB" | grep "PPP" > /dev/null	
	if [ $? -eq 0 ]; then
		if [ $PPP_MIB != YES ]; then
			echo "Enabling the PPP SNMP mib"
			WANPIPE_SNMP_MIBS="$WANPIPE_SNMP_MIBS ppp"
		fi
		MIB_MATCH=YES	
		PPP_MIB=YES
	fi

	echo "$MIB" | grep "X25" > /dev/null	
	if [ $? -eq 0 ]; then
		if [ $X25_MIB != YES ]; then
			echo "Enabling the X25 SNMP mib"
			WANPIPE_SNMP_MIBS="$WANPIPE_SNMP_MIBS x25"
		fi
		MIB_MATCH=YES	
		X25_MIB=YES
	fi

	echo "$MIB" | grep "DS1" > /dev/null	
	if [ $? -eq 0 ]; then
		if [ $DS1_MIB != YES ]; then
			echo "Enabling the DS1 SNMP mib"
			WANPIPE_SNMP_MIBS="$WANPIPE_SNMP_MIBS ds1 perfHistTCMIB"
		fi
		MIB_MATCH=YES	
		DS1_MIB=YES
	fi

	if [ $MIB_MATCH = NO ]; then
		echo
		echo "Error: invalid --mib option"
		echo
		echo "Available WANPIPE SNMP mibs: $WANPIPE_SNMP_ALL_MIBS";
		echo "Example:  ./wan_snmp_install.sh --mib=PPP-DS1"
		echo
		return 1
	fi

	return 0
}


function select_installation_mibs()
{
	local level=$1
	
	while [ 1 ];
	do

		banner

		cat <<ENDOFTEXT
WANPIPE SNMP MIBs
ENDOFTEXT

		if [ $FR_MIB = "NO" ]; then
			echo -n "[FR]   : frameRelayDTE mib	"
		else
			echo -n "[FR]*  : frameRelayDTE mib	"	
		fi
		if [ $PPP_MIB = "NO" ]; then
			echo    "[PPP]  : ppp mib" 	
		else
			echo    "[PPP]* : ppp mib" 
		fi
		if [ $X25_MIB = "NO" ]; then
			echo -n "[X25]  : X.25 mib		" 
		else
			echo -n "[X25]* : X.25 mib		" 
		fi
		if [ $DS1_MIB = "NO" ]; then
			echo    "[DS1]  : DS1 mib"
		else
			echo    "[DS1]* : DS1 mib"
		fi

	cat <<ENDOFTEXT

[ALL] : Enable All WANPIPE SNMP mibs  [NONE]: Disable ALL WANPIPE SNMP mibs

[EXIT|Q|q]: Exit WANPIPE SNMP mibs Selection

ENDOFTEXT

		echo -n "Please select a option in [<opt>]: "

		read response
	
		if [ "$response" = "EXIT" ]; then
			return 0;
		fi

		if [ "$response" = "Q" ]; then
			return 0;
		fi

		if [ "$response" = "q" ]; then
			return 0;
		fi

		if [ "$response" = "ALL" ]; then
			enable_snmp_mibs $WANPIPE_SNMP_ALL_MIBS
		elif [ "$response" = "NONE" ]; then
			init_all_mibs	
		else
			enable_snmp_mibs $response
			if [ $? -ne 0 ]; then
				pause
			fi
		fi

	done
}

# ----------------------------------------------------------------------------
# Apply kernel patches.
# ----------------------------------------------------------------------------
function copy_files()
{
	banner

	# Check src and dst directory.
	if [ ! -d "$WAN_SNMP_HOME/mibs" ]; then
		error "Failed to find WANPIPE SNMP mib files!"
		error "(Directory $WAN_SNMP_HOME/mibs not exists)!"
		return 1
	elif [ ! -d "$SNMP_HOME_DIR/$SNMP_MIBGROUP" ]; then
		error "Failed to find NET-SNMP mibgroup direct!"
		error "(Directory $SNMP_HOME_DIR/$SNMP_MIBGROUP not exists)!"
		return 1
	fi

	cd $WAN_SNMP_HOME
	for mib in $WANPIPE_SNMP_MIBS 
	do
		if [ $? -eq 0 ]; then
			case $mib in
			frameRelayDTE)
				cp mibs/frameRelayDTE.c	$SNMP_HOME_DIR/$SNMP_MIBGROUP
				cp mibs/frameRelayDTE.h	$SNMP_HOME_DIR/$SNMP_MIBGROUP
				;;
			ppp)
				cp mibs/ppp.c	$SNMP_HOME_DIR/$SNMP_MIBGROUP
				cp mibs/ppp.h	$SNMP_HOME_DIR/$SNMP_MIBGROUP
				;;
			ds1)
				cp mibs/ds1.c	$SNMP_HOME_DIR/$SNMP_MIBGROUP
				cp mibs/ds1.h	$SNMP_HOME_DIR/$SNMP_MIBGROUP
				;;
			perfHistTCMIB)
				cp mibs/perfHistTCMIB.c	$SNMP_HOME_DIR/$SNMP_MIBGROUP
				cp mibs/perfHistTCMIB.h	$SNMP_HOME_DIR/$SNMP_MIBGROUP
				;;
			x25) 
				cp mibs/x25.c	$SNMP_HOME_DIR/$SNMP_MIBGROUP
				cp mibs/x25.h	$SNMP_HOME_DIR/$SNMP_MIBGROUP
				;;
			*)
				echo "Unsupported WANPIPE SNMP protocol mib ($mib)"
				return 1
			esac
		fi
	done 

	# Create link from /usr/include/linux/wanpipe_snmp.h to 
	# /usr/src/linux/include/linux/wanpipe_snmp.h
	if [ $KERNEL_NAME = "Linux" ]; then 
		if [ ! -e "/usr/include/linux/wanpipe_snmp.h" ]; then
			ln -s /usr/src/linux/include/linux/wanpipe_snmp.h /usr/include/linux/wanpipe_snmp.h
		fi	
	fi
	return 0
}


function apply_patches()
{
	getyn "Would you like apply patches to $SNMP_VERSION?" || return 0
	cd $SNMP_HOME_DIR
	for file in $WANPIPE_SNMP_PATCH_FILES
	do
		eval "patch $SNMP_HOME_DIR/$SNMP_MIBII/$file $WAN_SNMP_HOME/patches/$file.patch.$SNMP_VERSION >/dev/null"
		if [ $? -ne 0 ]; then
			error "Failed to apply patch to $file"
			return 1
		fi 
	done

	return 0
}


function configure_snmp_daemon()
{

	cd $SNMP_HOME_DIR
	echo 
	eval "./configure $SNMP_OPTIONS --with-mib-modules=\"$WANPIPE_SNMP_MIBS\""
	if [ $? -eq 0 ]; then
		echo "Configuring SNMP daemon ...Done"
	else
		echo 
		error "Failed to configure SNMP daemon!"
		return 1
	fi
	return 0
}

function compily_install_snmp_daemon()
{
	cd $SNMP_HOME_DIR
	echo 
	echo -n "Compiling SNMP daemon .."
	eval "make clean >/dev/null"
	eval "make >/dev/null"
	if [ $? -eq 0 ]; then
		echo "Done"
		echo -n "Installing SNMP daemon ..."
		eval "make install >/dev/null"
		if [ $? -eq 0 ]; then
			echo "Done"
		else
			echo 
			error "Failed to install SNMP daemon!"
			return 1
		fi
	else
		echo 
		error "Failed to compile SNMP daemon!"
		return 1
	fi
	return 0
}

get_snmp_version ()
{
	cd $SNMP_HOME_DIR
	if [ ! -e $SNMPD ]; then
		# Configure, compile and install snmp daemon without any changes
		configure_snmp_daemon || exit 1
		compily_install_snmp_daemon || exit 1
	fi
	SNMP_VERSION=`$SNMPD --version | grep NET-SNMP | cut -d' ' -f4 2> /dev/null`
}

# ----------------------------------------------------------------------------
# Show welcome screen.
# ----------------------------------------------------------------------------
welcome()
{
	banner
	cat << ENDOFTEXT
WANPIPE SNMP mibs installation
	

You are about to install WANPIPE SNMP mibs file to SNMP daemon.

You must have the full NET-SNMP daemon source code in your system.
The default directory for NET-SNMP is $SNMP_HOME_DIR. If you installed
NET-SNMP daemon in a different directory, please specify this in a command line
(run ./$WAN_SNMP --help for more information).

This script will perform the following thins:
o Update and apply necessary patches to NET-SNMP daemon
o Configure NET-SNMP daemon
o Compile and install NET-SNMP daemon.

ENDOFTEXT
	getyn "Would you like to install WANPIPE SNMP mibs now?"
}

# ----------------------------------------------------------------------------
# Goodbye.
# ----------------------------------------------------------------------------
goodbye()
{
	banner
	cat << ENDOFTEXT
WANPIPE SNMP mibs installation: COMPLETE
	
WANPIPE SNMP mibs installation is no complete. NET-SNMP daemon configured,
compiled and installed.

If you will configure NET-SNMP daemon again, please do not forget include
necessary WANPIPE SNMP mibs to command line for configure script.

ENDOFTEXT


}


#############################################################
# set -x

PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/sbin
KERNEL_VERSION=`uname -r`
KERNEL_NAME=`uname -s`
WAN_SNMP=wan_snmp_install.sh
WAN_SNMP_VER=1.1.2
WAN_SNMP_HOME=`pwd`
SNMPD=./agent/snmpd
SNMP_HOME_DIR=/usr/local/net-snmp
SNMP_VERSION_FILE1=version.h
SNMP_VERSION_FILE2=snmplib/snmp_version.c
SNMP_MIBGROUP=agent/mibgroup
SNMP_MIBII=agent/mibgroup/mibII
SNMP_OPTIONS=
WANPIPE_SNMP_PATCH_FILES="interfaces.c"
#List of available non-standard protocols
WANPIPE_SNMP_ALL_MIBS="FR-DS1-PPP-X25"
WANPIPE_SNMP_MIBS=""

while [ ! -z "$1" ]
do
	arg=$1

	case $arg in
	--snmp_dir*)
		SNMP_HOME_DIR=`echo $arg | cut -d'=' -f2`
		;;
	--help*)
		print_help
		exit 0
		;;

	*)
		echo "Invalid option $arg"
		exit 1
		;;
	esac
	shift;
done

welcome || exit 1

# Get SNMP home directory
if [ ! -d $SNMP_HOME_DIR ]; then
	echo
	error "NET-SNMP home directory ($SNMP_HOME_DIR) is not exists!"
	error "Please, check the NET-SNMP directory and run this script again!"
	echo
	exit 0
fi

# Specify additional SNMP options
echo 
getyn "Do you want specify extra option for configuring SNMP daemon?"
if [ $? -eq 0 ]; then
	echo -e "\nPlease specify extra SNMP options >"
	read response
	SNMP_OPTIONS=$response
fi

init_all_mibs
get_snmp_version

select_installation_mibs || exit 1

copy_files || exit 1
apply_patches || exit 1

configure_snmp_daemon || exit 1
compily_install_snmp_daemon || exit 1

goodbye
exit 0

